#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package hostserv;

use strict;
use threads;
use threads::shared;

*get_user_id = \&nickserv::get_user_id;
*get_user_nick = \&nickserv::get_user_nick;
*is_identified = \&nickserv::is_identified;
*notice = \&nickserv::notice;

my $hsnick = 'HostServ';

my $err_deny = 'Permission denied.';

our (
	$set_vhost, $get_vhost, $del_vhost,
	$vhost_chgroot,

	$get_matching_vhosts
);

sub init() {
	$set_vhost = $services::dbh->prepare("REPLACE INTO vhost SET nick=?, ident=?, vhost=?, adder=?, time=UNIX_TIMESTAMP()");
	$get_vhost = $services::dbh->prepare("SELECT vhost.ident, vhost.vhost FROM vhost, nickalias WHERE nickalias.root=vhost.nick AND nickalias.alias=?");
	$del_vhost = $services::dbh->prepare("DELETE FROM vhost WHERE nick=?");

	$vhost_chgroot = $services::dbh->prepare("UPDATE vhost SET nick=? WHERE nick=?");

	$get_matching_vhosts = $services::dbh->prepare("SELECT nick, ident, vhost, adder, time FROM vhost WHERE nick LIKE ? AND ident LIKE ? AND vhost LIKE ? ORDER BY nick");
}

sub dispatch($$$) {
	my ($src, $dst, $msg) = @_;
	$msg =~ s/^\s+//;
	my @args = split(/\s+/, $msg);
	my $cmd = shift @args;

	my $user = { NICK => $src, AGENT => $hsnick };

	return if operserv::flood_check($user);

	if(lc $cmd eq 'on') {
		hs_on($user, $src, 1);
	}
	elsif(lc $cmd eq 'off') {
		hs_off($user);
	}
	elsif($cmd =~ /^(add|set(host))?$/i) {
		if (@args == 2) {
			hs_sethost($user, @args);
		}
		else {
			notice($user, 'Syntax: SETHOST <nick> <[ident@]vhost>');
		}
	}
	elsif($cmd =~ /^del(ete)?$/i) {
		if (@args == 1) {
			hs_delhost($user, @args);
		}
		else {
			notice($user, 'Syntax: DELETE <nick>');
		}
	}
	elsif($cmd =~ /^list$/i) {
		if (@args == 1) {
			hs_list($user, @args);
		}
		else {
			notice($user, 'Syntax: LIST <nick!vident@vhost>');
		}
	}	
        elsif($cmd =~ /^help$/i) {
                help::sendhelp($hsnick, $src, lc $hsnick, @args)
        }
	else { notice($user, "Unknown command."); }
}

sub hs_on($$;$) {
	my ($user, $nick, $flag) = @_;
	my $src = get_user_nick($user);
	
	unless(is_identified($user, $nick)) {
		notice($user, $err_deny);
		return;
	}
	
	$get_vhost->execute($nick);
	my ($vident, $vhost) = $get_vhost->fetchrow_array;
	unless ($vhost) {
		notice($user, "You don't have a vHost.") if $flag;
		return;
	}
	if ($vident) {
		net::chgident($hsnick, $src, $vident);
	}
	net::chghost($hsnick, $src, $vhost);

	notice($user, "Your vHost has been changed to \002".($vident?"$vident\@":'')."$vhost\002");
}

sub hs_off($) {
	my ($user) = @_;
	my $src = get_user_nick($user);
	
	net::setumode($hsnick, $src, '-xt');
	# doesn't work :(
	#net::setumode($hsnick, $src, '+x');
	
	notice($user, "vHost disabled.", "To enable normal host cloaking, /mode $src +x");
}

sub hs_sethost($$$) {
	my ($user, $target, $vhost) = @_;
	unless(adminserv::is_svsop($user, adminserv::S_OPER())) {
		notice($user, $err_deny);
		return;
	}
	my $rootnick = nickserv::get_root_nick($target);

	unless ($rootnick) {
		notice($user, "\002$target\002 is not registered.");
		return;
	}

	my $vident = '';
	if($vhost =~ /\@/) {
	    ($vident, $vhost) = split(/\@/, $vhost);
	}
	my $src = get_user_nick($user);
	$set_vhost->execute($rootnick, $vident, $vhost, $src);
	
	notice($user, "vHost for \002$target ($rootnick)\002 set to \002".($vident?"$vident\@":'')."$vhost\002");
}

sub hs_delhost($$) {
	my ($user, $target) = @_;
	unless(adminserv::is_svsop($user, adminserv::S_OPER())) {
		notice($user, $err_deny);
		return;
	}
	my $rootnick = nickserv::get_root_nick($target);

	unless ($rootnick) {
		notice($user, "\002$target\002 is not registered.");
		return;
	}

	$del_vhost->execute($rootnick);
	
	notice($user, "vHost for \002$target ($rootnick)\002 deleted.");
}

sub hs_list($$) {
	my ($user, $mask) = @_;

	unless(adminserv::is_svsop($user, adminserv::S_HELP())) {
		net::notice($user, $err_deny);
		return;
	}

	my ($mnick, $mident, $mhost) = nickserv::parse_mask($mask);

	$mnick = '%' if($mnick eq '');
	$mident = '%' if($mident eq '');
	$mhost = '%' if($mhost eq '');

	my @data;
	$get_matching_vhosts->execute($mnick, $mident, $mhost);
	while(my ($rnick, $vident, $vhost) = $get_matching_vhosts->fetchrow_array) {
		push @data, [$rnick, ($vident?"$vident\@":'').$vhost];
	}

	notice($user,
		"vHost list matching \002$mask\002:",
		fmt::columnar(@data)
	);
}


### MISCELLANEA ###

    
    
## IRC EVENTS ##

sub killhandle($$$$) {
	my ($src, $dst, $path, $reason) = @_;
	net::agent_connect('HostServ', 'services', undef, 'pqBS', 'vHost Agent')
		if (lc($dst) eq lc($hsnick));
}

1;
